""" Programming Language Zyku keywords
    by Tendhoya Kozi (kozi@tendhoya.jp)

「字空」字句～Python字句 対応の管理表

「字空」字句→Python字句への対応には次の2種類がある
  - 字句置き換え (例)文→str
    置き換えてpythonプログラムとして実行する
    delimiters, operators, keyword, builtin-type(class), methods of builtin-type(class)
  - wrapperによる実装
    置き換えずにpythonの識別子として日本語で定義（関数、クラス（類）など）し、そのまま実行する
    builtin-functions, wrappers for packages
"""

from enum import Flag, auto
from collections import namedtuple


class _Kind(Flag):
    """ 字句の種類
    """
    KEYWORD = auto()         # 予約語     '毎回'-> for, '返値'-> return  Zykuでは"match","case", "type"も予約語扱いとする
    DELIMITER = auto()       # 区切り文字 '「'-> "'"
    OPERATOR = auto()        # 演算子     '＋'-> +
    BL_TYPE = auto()         # 組込型       for builtin types     '文字列'-> str, '整数'-> int
    BL_FUNC = auto()         # 組込関数 for builtin functions replaced '全'-> all, '任'-> any
    BL_METHOD = auto()       # 組込型のメソッド for methods for builtin classes '加'-> obj.add, '付'-> obj.append
    WRAPPED_BL_FUNC = auto() # 組込関数のラッパー for builtin functions wrapped '停'-> breakpoint
    SPECIFIC_ID = auto()     # 特別な識別子　specific identifier '自'->self, '己'->'cls'
    WRAPPED_PKG = auto()     # パッケージのラッパー　for package wrappers '数'-> math, '乱'->random
    DUPLICATED = auto()      # 廃止　duplicated entry
    REPLACE = (              # Zyku->Pythonで置き換える対象
        KEYWORD | DELIMITER | OPERATOR | BL_TYPE | BL_FUNC | BL_METHOD | SPECIFIC_ID
    )  


_KwEntry = namedtuple('_KwEntry', [
    'py_kw',    # Python keyword
    'zyku_kw',  # Zyku Keyword
    'kind',     # class _Kind
    'comment'
    ])

_KwMasterList = [
    # _KwEntry(py_kw,        zyku_kw,  usage,  comment)
    # ========== 2.5. operators (longer than others) =====================
    _KwEntry("**=", "＊＊＝", _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry("**",  "＊＊",   _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry(",=",  "：＝",   _Kind.DELIMITER | _Kind.OPERATOR, "ウォルラス演算子"),
    _KwEntry("==",  "＝＝",   _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry("+=",  "＋＝",   _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry("-=",  "－＝",   _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry("*=",  "＊＝",   _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry("/=",  "／＝",   _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry("//=", "÷＝",   _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry("%=",  "％＝",   _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry("@=",  "＠＝",   _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry("&=",  "＆＝",   _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry("|=",  "｜＝",   _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry("^=",  "＾＝",   _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry(">>=", "≫＝",   _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry("<<=", "≪＝",   _Kind.DELIMITER | _Kind.OPERATOR, None),

    # ========== 2.5. operators =====================
    _KwEntry("+",   "＋", _Kind.OPERATOR, None),
    _KwEntry("-",   "－", _Kind.OPERATOR, None),
    _KwEntry("*",   "＊", _Kind.OPERATOR, "×（掛ける）はやめた ｘ、Ｘ（エックス）と間違えやすいので　注意。文脈によって意味が異なる（仮引数についた*は可変キーワード引数になる）"),     #
    # _KwEntry("**", "＊＊", _Kind.OPERATOR, None), # 上へ移動した
    _KwEntry("/",   "／",  _Kind.OPERATOR, None),
    _KwEntry("//",  "÷",  _Kind.OPERATOR, None),
    _KwEntry("%",   "％",  _Kind.OPERATOR, None),
    _KwEntry("@",   "＠",  _Kind.DELIMITER | _Kind.OPERATOR, None),
    _KwEntry("<<",  "≪",  _Kind.OPERATOR, None),
    _KwEntry(">>",  "≫",  _Kind.OPERATOR, None),
    _KwEntry("&",   "＆",  _Kind.OPERATOR, None),
    _KwEntry("|",   "｜",  _Kind.OPERATOR, None),
    _KwEntry("^",   "＾",  _Kind.OPERATOR, None),
    _KwEntry("~",   "～",  _Kind.OPERATOR, None),
    # _KwEntry(":=", "：＝", _Kind.OPERATOR, "ウォルラス演算子"), # 上へ移動した
    _KwEntry("<",   "＜",  _Kind.OPERATOR, None),
    _KwEntry(">",   "＞",  _Kind.OPERATOR, None),
    _KwEntry("<=",  "≦",  _Kind.OPERATOR, None),
    _KwEntry(">=",  "≧",  _Kind.OPERATOR, None),
    # _KwEntry("==", "＝＝", _Kind.OPERATOR, None), # 上へ移動した
    _KwEntry("!=",  "≠",  _Kind.OPERATOR, None),

    # ==========  2.6. delimiter =======================
    _KwEntry("(",   "（",  _Kind.DELIMITER, None),
    _KwEntry(")",   "）",  _Kind.DELIMITER, None),
    _KwEntry("[",   "［",  _Kind.DELIMITER, None),
    _KwEntry("]",   "］",  _Kind.DELIMITER, None),
    _KwEntry("{",   "｛",  _Kind.DELIMITER, None),
    _KwEntry("}",   "｝",  _Kind.DELIMITER, None),
    _KwEntry(",",   "，",  _Kind.DELIMITER, None),
    _KwEntry(":",   "：",  _Kind.DELIMITER, None),
    _KwEntry(".",   "．",  _Kind.DELIMITER, None),
    _KwEntry(";",   "；",  _Kind.DELIMITER, None),
    # _KwEntry("@", "＠",  _Kind.DELIMITER | _Kind.OPERATOR, None), # 重複あり
    _KwEntry("=",   "＝",  _Kind.DELIMITER, None),
    _KwEntry("->",  "→",  _Kind.DELIMITER, None),
    _KwEntry("...", "…",  _Kind.DELIMITER, "Ellipsis"),
    # :（コロン） ？で or に or は
    # ,（コンマ） 、

    # 個別対応
    # _KwEntry("__new__",      "＿新作＿", _Kind.BL_METHOD, None),  # 2
    _KwEntry("__init__",  "＿準備＿", _Kind.BL_METHOD, None),  # 4  備:4
    _KwEntry("__name__",  "＿名前＿", _Kind.BL_METHOD | _Kind.SPECIFIC_ID, None),  # :1 # 場 ＿名＿　＝＝ → if __name__ == にも適用される
    _KwEntry("__doc__",   "＿説明＿", _Kind.BL_METHOD, None),  # :4  document 説明
    _KwEntry("self",      "自",       _Kind.SPECIFIC_ID, None),  # :2 私:6 我:6 己:6 僕:常
    _KwEntry("cls",       "己",       _Kind.SPECIFIC_ID, None),  # 己:6 之,人名（之のクラス）
    # _KwEntry("'__main__'",  "主", _Kind.SPECIFIC_ID, None),  # :3 ←変換しない。実行環境側で入れるから。

    #  ==========  2.3.1. keyword =========================
    _KwEntry("False",    "いいえ", _Kind.KEYWORD, None),  #
    # _KwEntry("False",  "否",     _Kind.KEYWORD | _Kind.DUPLICATED, None),  # :6 (正否で対に) 不:4→not  偽,常用
    _KwEntry("None",     "無",     _Kind.KEYWORD, None),  # :4
    _KwEntry("True",     "はい",   _Kind.KEYWORD, None),  #
    # _KwEntry("True",   "正",     _Kind.KEYWORD | _Kind.DUPLICATED, None),  # :1 (正否で対) 真:3 （真偽）
    _KwEntry("and",      "で",     _Kind.KEYWORD, None),   # （「と」は曖昧 orの意味もあるため）   和:3→union 且,なし
    _KwEntry("as",       "を",     _Kind.KEYWORD, None),   # 名:1当:2 言:2（同はisで使った）如,なし,,みなす
    _KwEntry("assert",   "言明",   _Kind.KEYWORD, None),   # :5 鳴:2 泣:4 張:5 主張 表明（中国語は断言）断:5→break
    _KwEntry("async",    "裏方",   _Kind.KEYWORD, None),   # :6 （裏で動くから）並:6→iter.sort() 異:6 異歩(中文) 会:2 合:2  async def, async for, async with
    _KwEntry("await",    "待合",   _Kind.KEYWORD, None),   # :3
    _KwEntry("break",    "中断",   _Kind.KEYWORD, None),   # :5 去:3→iter.popitem() 出:1 卒:4(exit) 破:5 止:2(quit) 断:5(breakpoint) 絶:5 、抜,なし  脱
    _KwEntry("case",     "場合",   _Kind.KEYWORD, None),   # :2 Python v3.10
    _KwEntry("class",    "類型",   _Kind.KEYWORD, None),   # :4 (中国語で「類」)　級:3→classmethod  組:2→items
    _KwEntry("continue", "次回",   _Kind.KEYWORD, None),   # :4
    _KwEntry("def",      "定義",   _Kind.KEYWORD, None),   # :3
    _KwEntry("del",      "除去",   _Kind.KEYWORD, None),   # :3 →消:3 iter.clear() ★ハマった！2021/05/22
    _KwEntry("elif",     "別件",   _Kind.KEYWORD, None),   # :4 対:3 違,なし　もうひとつ
    _KwEntry("else",     "残件",   _Kind.KEYWORD, None),   # :3  if～elseに加え、for～elseをお忘れなく
    _KwEntry("except",   "除外",   _Kind.KEYWORD, None),   # :3 取:3 （例外を受ける）,別:4,非:5,除:6 外 異
    _KwEntry("finally",  "後始末", _Kind.KEYWORD, None),   # :3 後
    _KwEntry("for",      "毎回",   _Kind.KEYWORD, None),   # :2 各:4→map() 替,ない,向,,間はwhileで使った
    _KwEntry("from",     "引用",   _Kind.KEYWORD, None),   # :2 引:2→import 自:2,由 来
    _KwEntry("global",   "大域",   _Kind.KEYWORD, None),   # :2（公開）天:1？   外:2（対nonglobal 郡 内） 広:2だが狭は無い 皆（ない）全:3→all()（中国語では全局変量⇔local局部変量）永:5
    _KwEntry("if",       "条件",   _Kind.KEYWORD, None),   # 場:2（場合） 仮:5 時:2（→時間関係で使う）成:4 仮:5  もし（若）
    _KwEntry("import",   "取入",   _Kind.KEYWORD, None),   # :2（引用）使:3 入:1→insert
    _KwEntry("in",       "内",     _Kind.KEYWORD, None),   # :1 内:1 央:3→str.center
    _KwEntry("match",    "照合",   _Kind.KEYWORD, None),   # :4 Python v3.10
    _KwEntry("not in",   "不在",   _Kind.KEYWORD, None),   # 特殊対応 外:2→exception
    _KwEntry("is",       "は",     _Kind.KEYWORD, None),   # :2(as)之,なし  である、として　　仲間（間はwhile）仲:4 -> zip()
    _KwEntry("is not",   "でない", _Kind.KEYWORD, None),   # 特殊対応
    _KwEntry("lambda",   "式",     _Kind.KEYWORD, None),   # :3 評:5→eval()
    _KwEntry("nonlocal", "外側",   _Kind.KEYWORD, None),   # :4 郷里(local)の外側 外:2 外側のスコープの変数
    _KwEntry("not",      "反対",   _Kind.KEYWORD, None),   # :4 反:3→iter.reverse()（Falseでは使っていない）
    _KwEntry("or",       "か",     _Kind.KEYWORD, None),   #	又,なし  或いは
    _KwEntry("pass",     "通過",   _Kind.KEYWORD, None),   # :3 過:5  流:3  
    _KwEntry("raise",    "発出",   _Kind.KEYWORD, None),   # 起:3 挙:4 提:5, 投:3　上げる
    _KwEntry("return",   "返答",   _Kind.KEYWORD, None),   # :3
    _KwEntry("try",      "試行",   _Kind.KEYWORD, None),   # :4 観:4 察:4 確:5→assert  投:3
    _KwEntry("while",    "回続",   _Kind.KEYWORD, None),   # :2 繰,なし
    _KwEntry("with",     "使用",   _Kind.KEYWORD, None),   # :2 使:3 共:4→intersection
    _KwEntry("yield",    "明渡",   _Kind.KEYWORD, None),   # :2 渡,なし

    # ========== builtins replacement =======================
    _KwEntry("breakpoint",   "停止", _Kind.BL_FUNC, None),  # 停:4 断:5→break 止:1
    _KwEntry("exit",         "退出", _Kind.BL_FUNC, None),  # :4 脱,常
    _KwEntry("quit",         "中止", _Kind.BL_FUNC, None),  # :1 止める 断:5→break  終了
    _KwEntry("super",        "親方", _Kind.BL_FUNC, None),  # :2
    _KwEntry("classmethod",  "類方", _Kind.BL_FUNC, None),  # :3
    _KwEntry("staticmethod", "静方", _Kind.BL_FUNC, None),  # :4

    # ========== builtin type(class) ========================
    _KwEntry("bool",         "論理",   _Kind.BL_TYPE, None),   # :6 布:5  理:2  旗:4符,なし 表裏、（中文,布林）boolって人名よね
    _KwEntry('dict',         "辞書",   _Kind.BL_TYPE, None),   # :3 (対応表)3->dict.items 札と値の対、　組:2 -> zip 
    _KwEntry('float',        "実数",   _Kind.BL_TYPE, None),   # :3 数:2 実数 数←countと被る★（中文：浮点数）  小数にしようかとおもったが、小数にならない場合（整数）も含むので
    _KwEntry('frozenset',    "固集合", _Kind.BL_TYPE, None),
    _KwEntry('int',          "整数",   _Kind.BL_TYPE, None),   # :3 formatを形としてかぶらないようにした
    _KwEntry('list',         "配列",   _Kind.BL_TYPE, None),   # :3 連:4→range list->列表(中国語)
    _KwEntry("object",       "器",     _Kind.BL_TYPE, None),   # :4 物:3
    _KwEntry('range',        "範囲",   _Kind.BL_TYPE, None),   # :4しゃれ 範,常
    _KwEntry('set',          "集合",   _Kind.BL_TYPE, None),   # :3 組:2→zip
    _KwEntry('str',          "文字列", _Kind.BL_TYPE, None),   # :1 字:1→chr()
    _KwEntry('slice',        "断片",   _Kind.BL_TYPE, None),   # :6
    _KwEntry('tuple',        "束",     _Kind.BL_TYPE, None),   # :4
    _KwEntry('type',         "型",     _Kind.BL_TYPE, None),   # :4

    # ==========  methods in buildin classes dict,list,set,tuple,str ======
    _KwEntry('add',                 "追加",   _Kind.BL_METHOD, None),  # :4   set
    _KwEntry('append',              "付加",   _Kind.BL_METHOD, None),  # :4 付加 継,常   list
    _KwEntry('clear',               "消去",   _Kind.BL_METHOD, None),  # :4 消:3→del command   dict;list;set
    _KwEntry('copy',                "複写",   _Kind.BL_METHOD, None),  # :3   dict;list;set
    _KwEntry('count',               "個数",   _Kind.BL_METHOD, None),  # :5   list;tuple;str
    _KwEntry('difference',          "差集合", _Kind.BL_METHOD, None),  # :4   set
    _KwEntry('difference_update',   "差集合ノ改", _Kind.BL_METHOD, None),  # ,n        set
    _KwEntry('discard',             "取除",   _Kind.BL_METHOD, None),  # :6   set    #削除する
    _KwEntry('extend',              "結合",   _Kind.BL_METHOD, None),  # :6   list  #結合
    _KwEntry('fromkeys',            "引札",   _Kind.BL_METHOD, None),  # :2   dict  classmethod iter要素をキーにした新しいdictを作る。valueが指定されればその値が初期値になる。なければ値はNone(無)になる
    _KwEntry('get',                 "取得",   _Kind.BL_METHOD, None),  # :4   dict
    _KwEntry('index',               "位置",   _Kind.BL_METHOD, None),  # :4 番:2（何番にあるかを返すので）指:3 指数     list;tuple;str
    _KwEntry('insert',              "差入",   _Kind.BL_METHOD, None),  # ,常 挿,常 入:1→input()  突,常   list  #挿入
    _KwEntry('intersection',        "共通",   _Kind.BL_METHOD, None),  # :4  set    #積集合＝共通
    _KwEntry('intersection_update', "共通ノ改", _Kind.BL_METHOD, None),  # ,n      set
    _KwEntry('isdisjoint',          "素集合か", _Kind.BL_METHOD, None),  # :5   set #互いに素
    _KwEntry('issubset',            "部分か", _Kind.BL_METHOD, None),  # 部:3 部分集合  set  set.issubset(other)    set ⊆ other
    _KwEntry('issuperset',          "上位か", _Kind.BL_METHOD, None),  # 上:1 上位集合  含,常   set  set.issuperset(other)  set ⊇ other
    _KwEntry('items',               "品々",   _Kind.BL_METHOD, None),  # :2 (keyとvalueの組)  dict->対:3 組:2->zip
    _KwEntry('keys',                "札々",   _Kind.BL_METHOD, None),  # :4 鍵,常用でもない   dict
    _KwEntry('pop',                 "取出",   _Kind.BL_METHOD, None),  # :3 取り去る  dict;list;set
    _KwEntry('popitem',             "品出",   _Kind.BL_METHOD, None),  # :3 抜,常 dict
    _KwEntry('remove',              "取去",   _Kind.BL_METHOD, None),  # :6 削,常 割:6→divmod()  排,常 list;set
    _KwEntry('reverse',             "反転",   _Kind.BL_METHOD, None),  # :3   list ⇔ builtinsにreversed()→逆:5 もあるので注意
    _KwEntry('setdefault',          "初値",   _Kind.BL_METHOD, None),  # :4 既,常 default,既定値 "あれば値を返し無ければ値を入れる"  dict
    _KwEntry('sort',                "並変",   _Kind.BL_METHOD, None),  # :6   list ⇔ builtinsにsorted()→序:5 もあるので注意
    _KwEntry('symmetric_difference',"対称差", _Kind.BL_METHOD, None),  # ,常用 片:6→slice（奇(,常)数個ある場合...） 対称差    set
    _KwEntry('symmetric_difference_update', "対称差ノ改", _Kind.BL_METHOD, None),  # ,n      set
    _KwEntry('union',               "和集合", _Kind.BL_METHOD, None),  # 和:3 交,→zip 合:2→case    set    #和集合
    _KwEntry('update',              "改変",   _Kind.BL_METHOD, None),  # :4 更,常用   dict;set    #集合に足す s1.update(s2) s1とs2の和集合をs1とする
    _KwEntry('values',              "値々",   _Kind.BL_METHOD, None),  # :6   dict
    # view->'景'

    # ========== methods in buildin class str ==================
    # 'capitalize',   "", #,n   str #最初を大文字に
    # 'casefold',     "", #,n   str
    _KwEntry('center',       "中寄", _Kind.BL_METHOD, None),  # 寄:5  央:3 str   "中央寄せ"
    # 'encode',       "", #,n   str　"バイト列に"
    _KwEntry('endswith',     "文末", _Kind.BL_METHOD, None),  # :4   str　"終わりは"
    # 'expandtabs',   "", #,n   str　"タブ文字を空白に"
    _KwEntry('find',         "探す", _Kind.BL_METHOD, None),  # 探:6 見:1つける    str
    _KwEntry('format',       "整形", _Kind.BL_METHOD, None),  # :2   str  #書式化 整, →int
    # 'format_map',   "", #,n   str Python v3.2
    _KwEntry('isalnum',      "英数か", _Kind.BL_METHOD, None),  # 英:4   str "英数文字か"
    _KwEntry('isalpha',      "英字か", _Kind.BL_METHOD, None),  # 英:4   str "英文字か"
    # 'isascii',      "", #,n   str  Python v3.7以降
    _KwEntry('isdecimal',    "十進か", _Kind.BL_METHOD, None),  # ,n   str  "全て数字か"'12345'.isdecimal()→True, '12.345'.isdecimal()→False
    _KwEntry('isdigit',      "数値か", _Kind.BL_METHOD, None),  # ,n   str "数字か"'①'.isdigit()→True
    _KwEntry('isidentifier', "名前か", _Kind.BL_METHOD, None),  # ,n   str "名前か"
    _KwEntry('islower',      "小字か", _Kind.BL_METHOD, None),  # ,n   str "小文字か"
    _KwEntry('isnumeric',    "数字か", _Kind.BL_METHOD, None),  # ,n   str "数を表す文字か"  '三'.isnumeric()→True, '円'.isnumeric()→False
    _KwEntry('isprintable',  "印字か", _Kind.BL_METHOD, None),  # ,n   str "印字可能な文字か"
    _KwEntry('isspace',      "空白か", _Kind.BL_METHOD, None),  # ,n   str "空白文字か"
    _KwEntry('istitle',      "題字か", _Kind.BL_METHOD, None),  # ,題:3   str
    _KwEntry('isupper',      "大字か", _Kind.BL_METHOD, None),  # ,n   str "大文字か"
    _KwEntry('join',         "連結",   _Kind.BL_METHOD, None),  # :4   str
    _KwEntry('ljust',        "左寄",   _Kind.BL_METHOD, None),  # 寄:5   str "左揃"
    _KwEntry('lower',        "小字化", _Kind.BL_METHOD, None),  # ,n   str "小文字化"
    _KwEntry('lstrip',       "頭除",   _Kind.BL_METHOD, None),  # 落:3 strip->trim切り落とす、刈り込み、手入れ  右:1 身:3   str "先頭の空白を除去"
    # 'maketrans',    "", #,n  str str.translate()用の変換テーブルをつくる
    _KwEntry('partition',    "区切",   _Kind.BL_METHOD, None),  # :3   str "文字列を区切る"
    _KwEntry('replace',      "交代",   _Kind.BL_METHOD, None),  # :3 替,常 換,常 str "置換"
    _KwEntry('removeprefix', "頭切",   _Kind.BL_METHOD, None),  # 
    _KwEntry('removesuffix', "末切",   _Kind.BL_METHOD, None),  # 
    _KwEntry('rfind',        "末探",   _Kind.BL_METHOD, None),  # ,n   str "右から探す"
    _KwEntry('rindex',       "末番",   _Kind.BL_METHOD, None),  # ,n   str "右から探す。見つからないときはValueErrorを出す"
    _KwEntry('rjust',        "右寄",   _Kind.BL_METHOD, None),  # 寄:5   str "右寄せ"
    _KwEntry('rpartition',   "右区切", _Kind.BL_METHOD, None),  # ,n   str "文字列を区切る（右から探す）"
    _KwEntry('rsplit',       "逆分割", _Kind.BL_METHOD, None),  # ,n   str "文字列を分割する（右から探す）"
    _KwEntry('rstrip',       "末除",   _Kind.BL_METHOD, None),  # 落:3 strip->trim切り落とす、刈り込み、手入れ  左:1 身:3   str "末尾の空白を除去"
    _KwEntry('split',        "分割",   _Kind.BL_METHOD, None),  # :2   str "文字列を分割する"
    _KwEntry('splitlines',   "行切",   _Kind.BL_METHOD, None),  # ,n   str "改行文字で分割する"
    _KwEntry('startswith',   "文頭",   _Kind.BL_METHOD, None),  # :3   str
    _KwEntry('strip',        "空除",   _Kind.BL_METHOD, None),  # :3 strip->trim切り落とす、刈り込み、手入れ   剥,常 身:3 囲:4   str "先頭末尾の空白文字を除去する"  剥,常用でもない
    # 'swapcase',     "", #,n   str "大文字⇔小文字入れ替え"
    # 'title',        "", #,n   str "先頭を大文字に"
    # 'translate',    "", #,n   str "文字を置き換える" 辞書はstr.maketrans()で作る
    _KwEntry('upper',        "大字化", _Kind.BL_METHOD, None),  # ,n   str "大文字化する"
    # 'zfill',        ""  #,常   str "ゼロで埋める"

    # ========== wrappers.builtins ==================================
    _KwEntry('abs',       "絶対値",       _Kind.WRAPPED_BL_FUNC, None),   # math.fabs() -> 絶対値
    _KwEntry('all',       "全て",       _Kind.WRAPPED_BL_FUNC, None),
    _KwEntry('any',       "どれか",       _Kind.WRAPPED_BL_FUNC, None),
    # _KwEntry('ascii',     "アスキー", _Kind.WRAPPED_BL_FUNC, None),
    _KwEntry('bin',       "二進",   _Kind.WRAPPED_BL_FUNC, None),
    # _KwEntry("bool",      "論理", _Kind.BL_TYPE, None),   # :6 布:5  理:2  旗:4符,なし 表裏、（中文,布林）boolって人名よね
    # _KwEntry("breakpoint",   "停", _Kind.BL_FUNC, None),  # 停:4 断:5→break 止:1
    # 'bytearray': 'バイト配列',
    # 'bytes':     'バイト列',
    # 'callable':  '呼び出し可能な',
    _KwEntry('chr',       "文字",       _Kind.WRAPPED_BL_FUNC, None),
    # _KwEntry("classmethod",  "類ノ法", _Kind.BL_FUNC, None),  # :3
    # 'compile':   '解読',
    _KwEntry('complex',   "複素数",       _Kind.WRAPPED_BL_FUNC, None),
    # 'delattr':   '属性を削除する',
    # _KwEntry('dict',    "対", _Kind.BL_TYPE, None),   # :3 (対応表)3->dict.items 札と値の対、　組:2 -> zip 
    _KwEntry('dir',       "目録",       _Kind.WRAPPED_BL_FUNC, None),
    _KwEntry('divmod',    "商と余",     _Kind.WRAPPED_BL_FUNC, None),
    _KwEntry('enumerate', "順番",       _Kind.WRAPPED_BL_FUNC, None), # :2 順:4 位:4→index
    # 'eval':      '式を計算する',  #式:3→lambda   #広: 狭: 域:6 #問題：字空文法に対応していない。翻訳後 eval()を呼ぶべき
    # 'exec':      '実行する',  #あえて実装しない（セキュリティー）
    # 'execfile':  'ファイルを実行する', #あえて実装しない（セキュリティー）
    _KwEntry('filter',    "選別",       _Kind.WRAPPED_BL_FUNC, None),
    # _KwEntry('float',   "実", _Kind.BL_TYPE, None),   # :3 数:2 実数 数←countと被る★（中文：浮点数）
    # 'format':    '形', #:2 intを整としてかぶらないようにした  str.formatと被るが、漢字が違うなら機能割り当て可能。字空文法に対応させる
    # _KwEntry('frozenset',       "固ノ集",       _Kind.BL_TYPE, None),
    # 'getattr':   '属性の値',
    # 'globals':   '公変数',
    # 'hasattr':   '属性を持つ',
    # 'hash':      'ハッシュ値',
    _KwEntry('help',      "手伝",       _Kind.WRAPPED_BL_FUNC, None),
    _KwEntry('hex',       "十六進",   _Kind.WRAPPED_BL_FUNC, None),
    _KwEntry('id',        "識別",       _Kind.WRAPPED_BL_FUNC, None),
    _KwEntry('input',     "入力",       _Kind.WRAPPED_BL_FUNC, None),
    # _KwEntry('int',     "整",       _Kind.BL_TYPE, None),   # :3 formatを形としてかぶらないようにした
    # 'isinstance': '実体',
    # 'issubclass': '子供',
    # 'iter':       'くり返しオブジェクト', #繰:常
    _KwEntry('len',       "長さ",       _Kind.WRAPPED_BL_FUNC, None),
    # _KwEntry('list',    "列",       _Kind.BL_TYPE, None),   # :3 連:4→range list->列表(中国語)
    # 'locals':   '変数の列',
    # 'map':      'それぞれを処理', 各:4 毎:2→for   キーワード置き換えで対応
    _KwEntry('map',       "各項",       _Kind.WRAPPED_BL_FUNC, None),   # :4
    _KwEntry('max',       "最大値",   _Kind.WRAPPED_BL_FUNC, None),
    # 'memoryview': 'メモリビュー', #C言語I/F
    _KwEntry('min',       "最小値",   _Kind.WRAPPED_BL_FUNC, None),
    _KwEntry('next',      "次品",       _Kind.WRAPPED_BL_FUNC, None),
    # _KwEntry("object",  "器",       _Kind.BL_TYPE, None),   # :4 物:3
    # 'oct':      '八進',
    _KwEntry('open',      "開く",       _Kind.WRAPPED_BL_FUNC, None),
    _KwEntry('ord',       "字番",       _Kind.WRAPPED_BL_FUNC, None),
    _KwEntry('pow',       "冪乗",       _Kind.WRAPPED_BL_FUNC, None),
    _KwEntry('print',     "印字",       _Kind.WRAPPED_BL_FUNC, None),
    # 'property': '属' プロパティ,
    # _KwEntry('range',        "連",  _Kind.BL_TYPE, None),   # :4しゃれ 範,常
    _KwEntry('repr',      "表現",       _Kind.WRAPPED_BL_FUNC, None),
    _KwEntry('reversed',  "逆順",       _Kind.WRAPPED_BL_FUNC, None),
    _KwEntry('round',     "丸める",       _Kind.WRAPPED_BL_FUNC, None),
    # 'runfile':  '実行ファイル',
    # _KwEntry('set',     "集",       _Kind.BL_TYPE, None),   # :3 組:2→zip
    # 'setattr':  '属性に書き込む',
    # _KwEntry('slice',   "片",       _Kind.BL_TYPE, None),   # :6
    _KwEntry('sorted',    "整列",       _Kind.WRAPPED_BL_FUNC, None),
    # _KwEntry("staticmethod", "静ノ法", _Kind.BL_FUNC, None),  # :4
    # _KwEntry('str',     "文",       _Kind.BL_TYPE, None),   # :1 字:1→chr()
    _KwEntry('sum',       "合計",       _Kind.WRAPPED_BL_FUNC, None),
    # _KwEntry("super",   "親",       _Kind.BL_FUNC, None),  # :2
    # _KwEntry('tuple',   "束",       _Kind.BL_TYPE, None),   # :4
    # _KwEntry('type',    "型",       _Kind.BL_TYPE, None),   # :4
    # 'vars':     '変数の列',
    _KwEntry('zip',       "接合",       _Kind.WRAPPED_BL_FUNC, None),
    # __import__()  あえて乗せる必要なし

    # class int, float内に、あえて日本語化するような（初心者が使うような）メソッドは無い

    _KwEntry("_",            "＿", _Kind.KEYWORD, None),  # to replace, treat '_' as Keyword in Zyku, for match~case _, Python v3.10
]


def zy_to_py_replacement_dict() -> dict[str, str]:
    """ 「字空」字句→Python字句 変換辞書を返す """
    zy_py_dict = {itm.zyku_kw: itm.py_kw
                  for itm in _KwMasterList if itm.usage & _Kind.REPLACE}
    zy_py_dict["　"] = "  "     # 全角スペース1文字→半角スペース2文字  zy->py のみ入れる
    return zy_py_dict


def py_to_zy_replacement_dict() -> dict[str, str]:
    """ Python字句→「字空」字句 変換辞書を返す """
    py_zy_dict =  {
        itm.py_kw: itm.zyku_kw \
            for itm in _KwMasterList \
            if (itm.usage & _Kind.REPLACE and not (itm.usage & _Kind.DUPLICATED))
    }

    return py_zy_dict


def check_keywords() -> bool:
    """ 字句変換表の重複チェッカー 
    
    Returns:
        (bool) True 重複なし。おめでとうメッセージを印字する
               False 重複あり。重複しているエントリを印字する    
    """
    duplicates = []
    zy_keywords = set()
    py_keywords = set()

    for itm in _KwMasterList:
        if itm.zyku_kw in zy_keywords:
            # 前にある同キーワードのエントリを探して足す
            for olditm in  _KwMasterList:
                if olditm.zyku_kw == itm.zyku_kw:
                    duplicates.append(olditm)
                    break

            # 今回見つかったものを足す
            duplicates.append(itm)
        
        if itm.py_kw in py_keywords:
            for olditm in  _KwMasterList:
                if olditm.py_kw == itm.py_kw:
                    duplicates.append(olditm)
                    break

            # 今回見つかったものを足す
            duplicates.append(itm)

        zy_keywords.add(itm.zyku_kw)
        py_keywords.add(itm.py_kw)

    if len(duplicates) == 0:
        print("No duplicated entries, congraturations!")
        return True

    print(f" {len(duplicates)} duplicated entries founded.")
    for idx, dupitem in enumerate(duplicates, start=1):
        print(idx, dupitem)

    return False


if __name__ == "__main__":
    check_keywords()
